#!/usr/bin/env python
# -*- coding: utf-8

"""Singleton configuration for k8s client"""

api_server = "https://kubernetes.default.svc.cluster.local"
api_token = ""
cert = None
verify_ssl = True
debug = False
